/*
*	TechFactory.hu
*	Az IGAZI Gomb 
*	2017 - 2019
*/

#include <ESP8266WiFi.h> 
#include <WiFiUdp.h>
#include <ESP8266HTTPClient.h>
#include <Adafruit_NeoPixel.h>

#define TF_DEBUG 1;

int sessid = 0;
unsigned long ledActionTime = 0;
unsigned long lastStatusTime = 0;
unsigned long waitTime = 0;

const String wifiSSID = "T-Factory";
const String wifiPasswd = "**************";

int status = 0;
HTTPClient http;

Adafruit_NeoPixel pixels;

struct RGB {
  uint8_t r;
  uint8_t g;
  uint8_t b;
};

uint8_t ledType = 0;
struct LedStatus {
   uint8_t led;
   uint8_t type;
   unsigned long change;
};

LedStatus lStatus = {0,0,0};

#define BUTTON_PIN 4
#define POWER_PIN 13

void setup() {
    #ifdef TF_DEBUG
    Serial.begin(115200);
    #endif 
    
    pinMode(POWER_PIN, OUTPUT);
    pinMode(BUTTON_PIN, INPUT_PULLUP);
    digitalWrite(POWER_PIN,LOW);
    
    #ifdef TF_DEBUG
    Serial.println("Connecting to wifi ap");
    #endif
    
    WiFi.begin(wifiSSID, wifiPasswd);
    generateSessid();

    #ifdef TF_DEBUG
    Serial.println("Session id is "+String(sessid));
    #endif
    
    pixels.updateType(NEO_GRB + NEO_KHZ800);
    pixels.setPin(14);
    pixels.updateLength(4);
    pixels.begin();

    for (int i=0; i<4; i++) {
      pixels.setPixelColor(i,0,255,50*i);
    }
    
    pixels.show();
    ledType = 1;
}

void loop() {
  if (digitalRead(BUTTON_PIN) == 0) {
      if (status == -1) {
        #ifdef TF_DEBUG
        Serial.println("Restart requested");
        #endif
        status = 0;
        waitTime = 0;
      }else {
        //wait
        ledType = 1;
      }
  }
  
  //wait for network
  if (WiFi.status() != WL_CONNECTED) {
     if (millis() > 20000) {
          ledType = 4;  
     }else {
          ledType = 1;
     }
  }else {
    //request a call
    if (status == 0) {
        ledType = 5;
        ledDisplay();
        if (sendRequest(1) == 1) {
          ledType = 2;
          status = 1;
          lastStatusTime = millis();
        }else {
          status = -1;
          ledType = 4;  
          waitAndTurnOff(30000);
        }
    }else if(status == 1) {
        if (lastStatusTime + 3000 < millis()) {
          int callstatus = sendRequest(2);
          //done
          if (callstatus == 0) {
             #ifdef TF_DEBUG
             Serial.println("Call was successful");
             #endif
             
             ledType = 3;
             waitAndTurnOff(10000);
             status = 2;
          //wait
          }else if(callstatus == 1) {
             #ifdef TF_DEBUG
             Serial.println("Call in progress");
             #endif
             
             ledType = 2;
          }else if(callstatus == 3) {
               status = -1;
               ledType = 4;  
               waitAndTurnOff(10000);
          }
          
          lastStatusTime = millis();
        }
    }
  }
  
  ledDisplay();

  //max runtime
  if (millis() > 120000) {
      turnOff();
  }
  
  waitAndTurnOff(0);
}

void ledDisplay() {
    if (ledActionTime + 100 < millis()) {
      if (lStatus.type != ledType) {
          #ifdef TF_DEBUG
            Serial.println("[Pixel] Type changed to "+String(ledType));
          #endif
          lStatus.led = 0;
          lStatus.type = ledType;
          lStatus.change = 0;
      }
      //wifi
      if (ledType == 1) {
          if (lStatus.led == 0 || lStatus.change+500 < millis()) {
              if (lStatus.led == 0) { lStatus.led = 1; }
              for (int i=0; i<4;i++) {
                 if (i == lStatus.led-1) {
                     pixels.setPixelColor(i,0,100,255); 
                 }else {
                     pixels.setPixelColor(i,0,0,0); 
                 }
              }
              lStatus.led++;
              if (lStatus.led == 5) { lStatus.led = 1; }
              lStatus.change = millis();
              pixels.show();
          }
      }
      //call in progress
      else if (ledType == 2) {
           if (lStatus.led == 0 || lStatus.change+200 < millis()) {
              if (lStatus.led == 0) { lStatus.led = 1; }
              pixels.setPixelColor((lStatus.led-1),0,255,150); 
              int px = 0;
              for (int i=1; i<4;i++) {
                  px = lStatus.led-1-i;
                  if (px < 0) {  px = px + 4; }
                  pixels.setPixelColor(px,0,round(255*0.25*(4-i)),round(  150*0.25*(4-i))); 
              }

              lStatus.led++;
              if (lStatus.led == 5) { lStatus.led = 1; }
              
              lStatus.change = millis();
              pixels.show();
          }
      }
      //done
      else if (ledType == 3) {
           if (lStatus.led == 0 || lStatus.change+300 < millis()) {
              if (lStatus.led == 0) { lStatus.led = 1; }
              for (int i=0; i<4;i++) {
                  if (lStatus.led == 1) {  pixels.setPixelColor(i,0,255,0); }
                  else {
                     pixels.setPixelColor(i,0,0,0);
                  }
              }
              if (lStatus.led == 1) { lStatus.led = 2; }
              else { lStatus.led = 1; }
              lStatus.change = millis();
              pixels.show();
          }
      }
      //error
      else if (ledType == 4) {
           float p;
           if (lStatus.led == 0) {
              if (lStatus.change == 0) { lStatus.change = millis(); }
              p = (float)(millis() - lStatus.change) / (float)1000;

              if (p <= 1) {
                fadeColor(0, RGB{0,0,0}, RGB{255,0,0}, p);
                fadeColor(2, RGB{0,0,0}, RGB{255,0,0}, p);
                fadeColor(1, RGB{0,0,0}, RGB{200,0,100}, p);
                fadeColor(3, RGB{0,0,0}, RGB{200,0,100}, p);
              }else {
                 lStatus.led = 1;
                 lStatus.change = millis();
              }
           }else if(lStatus.led == 1) {
              p = (float)(millis() - lStatus.change) / (float)2000;
              if (p <= 1) {
                fadeColor(0, RGB{255,0,0}, RGB{200,0,100}, p);
                fadeColor(2, RGB{255,0,0}, RGB{200,0,100}, p);
                fadeColor(1, RGB{200,0,100}, RGB{255,0,0}, p);
                fadeColor(3, RGB{200,0,100}, RGB{255,0,0}, p);
              }else {
                lStatus.led = 2;
                lStatus.change = millis();
              }
           }else if(lStatus.led == 2) {
              p = (float)(millis() - lStatus.change) / (float)2000;
              if (p <= 1) {
                fadeColor(0, RGB{200,0,100}, RGB{20,0,0}, p);
                fadeColor(2, RGB{200,0,100}, RGB{20,0,0}, p);
                fadeColor(1, RGB{255,0,0}, RGB{20,0,5}, p);
                fadeColor(3, RGB{255,0,0}, RGB{20,0,5}, p);
              }else {
                lStatus.led = 3;
                lStatus.change = millis();
              }
           }
           else if(lStatus.led == 3) {
              p = (float)(millis() - lStatus.change) / (float)2000;
              if (p <= 1) {
                fadeColor(0, RGB{20,0,0}, RGB{255,0,0}, p);
                fadeColor(2, RGB{20,0,0}, RGB{255,0,0}, p);
                fadeColor(1, RGB{20,0,5}, RGB{200,0,100}, p);
                fadeColor(3, RGB{20,0,5}, RGB{200,0,100}, p);
              }else {
                lStatus.led = 1;
                lStatus.change = millis();
              }
           }
            pixels.show();
      }
  
      else if (ledType == 5) {
            if (lStatus.led == 0) {
                if (lStatus.led == 0) { lStatus.led = 1; }
                for (int i=0; i<4;i++) {
                    pixels.setPixelColor(i,0,0,255); 
                }
                pixels.show();
            }
      }

      ledActionTime = millis();
    }
}


void fadeColor(int pxid, RGB c1, RGB c2, float p) {
    RGB out;
    out.r = _FadeRGBHelper(c1.r,c2.r ,p);
    out.g = _FadeRGBHelper(c1.g,c2.g ,p);
    out.b = _FadeRGBHelper(c1.b,c2.b ,p);
    pixels.setPixelColor(pxid, out.r, out.g, out.b);
}

int _FadeRGBHelper(int s, int e, float p) {
    int diff = s - e;
    return diff != 0 ?  constrain( ceil(s - (diff * p)),0,255) : s;
}

int sendRequest(int action) {
    #ifdef TF_DEBUG
    Serial.println("Request action is "+String(action));
    #endif
    
    http.begin("http://192.168.1.112/?sessid="+String(sessid)+"&action="+String(action));
    int httpCode = http.GET();
    int stcode = 0;
    if(httpCode > 0) {
        String payload = http.getString();
        stcode = payload.toInt();
        http.end();
        #ifdef TF_DEBUG
        Serial.println("> "+String(stcode));
        #endif
        return stcode;
    }
    return -1;
}

void generateSessid() {
  sessid = random(1,5000);
}

void waitAndTurnOff(int wait) {
  if (waitTime == 0 && wait > 0) {
      #ifdef TF_DEBUG
      Serial.println("Turning off after "+String(wait)+" ms");
      #endif
      
      waitTime = millis() + wait;
  }else if (waitTime > 0 && millis() > waitTime) {
       #ifdef TF_DEBUG  
    Serial.println("Wait time elapsed. ");
   #endif
      turnOff();
  }
}

void turnOff() {
   #ifdef TF_DEBUG  
      Serial.println("Turning off. Good bye!");
   #endif
   for (int i=0; i<4;i++) {
      pixels.setPixelColor(i,0,0,0); 
   }
   pixels.show();
   delay(100);
   pinMode(POWER_PIN, INPUT_PULLUP);
   delay(100);
}
